class NotHangleException(Exception):
    def __init__(self):
        super().__init__("마지막 글자가 한글이 아닙니다.")

class JosaTypeException(Exception):
    def __init__(self):
        super().__init__("메서드의 인자로 주어진 조사가 올바르지 않습니다.")

class JongsungInstantiationException(Exception):
    def __init__(self):
        super().__init__("종성 클래스는 인스턴스화 할 필요가 없습니다.")


#-*- coding: utf-8 -*-
import re
from pyjosa.exceptions import NotHangleException, JongsungInstantiationException


class Jongsung:
    """
    글자가 한글인지 체크 및 종성이 있는지 체크하는 클래스
    """
    # Class attribute
    _START_HANGLE = 44032
    _J_IDX = 28

    # we will not instantiate this class because it's not really needed.
    def __init__(self):
        raise JongsungInstantiationException

    @staticmethod
    def is_hangle(string: str) -> bool:
        """
        입력받은 문자열의 마지막 글자가 한글인지 체크하는 정적 메서드
        :param string: 처리하고자 하는 단어(문자열)
        :return: 마지막 글자가 한글일경우 True, 한글이 아닐경우 False
        """
        last_char = string[-1]
        if re.match('.*[ㄱ-ㅎㅏ-ㅣ가-힣]+.*', last_char) is None:
            return False
        return True

    @classmethod
    def has_jongsung(cls, string: str) -> bool:
        """
        입력받은 문자열의 마지막 글자가 한글인 경우, 해당 글자가 종성이 있는지 없는지 확인하는
        클래스 메서드
        :param string: 입력받은 문자열
        :return: 종성이 있을경우 True
        """
        if not cls.is_hangle(string):
            raise NotHangleException
        last_char = string[-1]
        if (ord(last_char) - cls._START_HANGLE) % cls._J_IDX > 0:
            return True
        return False



from pyjosa.jonsung import Jongsung
from pyjosa.exceptions import JosaTypeException


class Josa:

    @staticmethod
    def get_josa(string: str, josa: str) -> str:
        """
        조사를 반환하는 정적 메서드
        :param string: 입력받는 한글 단어
        :param josa: 체크하고자 하는 조사
        :return: 종성 여부에 따라 적절한 조사 반환
        """

        if (josa == '을') or (josa == '를'):
            return '을' if Jongsung.has_jongsung(string) else '를'
        elif (josa == '은') or (josa == '는'):
            return '은' if Jongsung.has_jongsung(string) else '는'
        elif (josa == '이') or (josa == '가'):
            return '이' if Jongsung.has_jongsung(string) else '가'
        elif (josa == '과') or (josa == '와'):
            return '과' if Jongsung.has_jongsung(string) else '와'
        elif (josa == '이나') or (josa == '나'):
            return '이나' if Jongsung.has_jongsung(string) else '나'
        elif (josa == '으로') or (josa == '로'):
            return '으로' if Jongsung.has_jongsung(string) else '로'
        elif (josa == '아') or (josa == '야'):
            return '아' if Jongsung.has_jongsung(string) else '야'
        elif (josa == '이랑') or (josa == '랑'):
            return '이랑' if Jongsung.has_jongsung(string) else '랑'
        elif (josa == '이며') or (josa == '며'):
            return '이며' if Jongsung.has_jongsung(string) else '며'
        elif (josa == '이다') or (josa == '다'):
            return '이다' if Jongsung.has_jongsung(string) else '다'
        elif josa == '이가':
            return '이가' if Jongsung.has_jongsung(string) else '가'
        else:
            raise JosaTypeException

    @staticmethod
    def get_full_string(string: str, josa: str) -> str:
        """
        단어 뒤에 조사를 붙여서 반환하는 정적 메서드
        :param string: 입력받는 한글 단어
        :param josa: 체크하고자 하는 조사
        :return: 단어와 조사를 붙인 문자열을 반환
        """
        if (josa == '을') or (josa == '를'):
            return string + '을' if Jongsung.has_jongsung(string) else string + '를'
        elif (josa == '은') or (josa == '는'):
            return string + '은' if Jongsung.has_jongsung(string) else string + '는'
        elif (josa == '이') or (josa == '가'):
            return string + '이' if Jongsung.has_jongsung(string) else string + '가'
        elif (josa == '과') or (josa == '와'):
            return string + '과' if Jongsung.has_jongsung(string) else string + '와'
        elif (josa == '이나') or (josa == '나'):
            return string + '이나' if Jongsung.has_jongsung(string) else string + '나'
        elif (josa == '으로') or (josa == '로'):
            return string + '으로' if Jongsung.has_jongsung(string) else string + '로'
        elif (josa == '아') or (josa == '야'):
            return string + '아' if Jongsung.has_jongsung(string) else string + '야'
        elif (josa == '이랑') or (josa == '랑'):
            return string + '이랑' if Jongsung.has_jongsung(string) else string + '랑'
        elif (josa == '이며') or (josa == '며'):
            return string + '이며' if Jongsung.has_jongsung(string) else string + '며'
        elif (josa == '이다') or (josa == '다'):
            return string + '이다' if Jongsung.has_jongsung(string) else string + '다'
        elif josa == '이가':
            return string + '이가' if Jongsung.has_jongsung(string) else string + '가'
        else:
            raise JosaTypeException

# TODO : Refactor pyjosa's architecture with oop.
# TODO : need to remove duplicated codes with 'if ... elif...' (refactor)
